GSSAPI="BASE"  # This ensures that a full module is generated by Cython

from gssapi.raw.cython_types cimport *
from gssapi.raw.cython_converters cimport c_make_oid

from gssapi.raw import types as gsstypes

"""Kerberos-specific constants

Upon import, this module will populate
Kerberos-specific constants into NameType
and MechType.
"""


cdef extern from "python_gssapi_krb5.h":
    gss_OID gss_mech_krb5
    gss_OID GSS_KRB5_NT_PRINCIPAL_NAME

gsstypes.NameType.kerberos_principal = c_make_oid(GSS_KRB5_NT_PRINCIPAL_NAME)
gsstypes.MechType.kerberos = c_make_oid(gss_mech_krb5)
