GSSAPI="BASE"  # This ensures that a full module is generated by Cython

from gssapi.raw.cython_types cimport gss_OID
from gssapi.raw.cython_converters cimport c_make_oid

from gssapi.raw import types as gsstypes


# NB(directxman12): this is placed in separate file since the
# GSS_C_NT_COMPOSITE_EXPORT constant didn't appear in MIT
# krb5 until 1.11.  However, due to the way that support was
# written for composite tokens, simply using GSS_C_NT_EXPORT_NAME
# will work in prior version which contain support for RFC 6680
cdef extern from "python_gssapi_ext.h":
    gss_OID GSS_C_NT_COMPOSITE_EXPORT


gsstypes.NameType.composite_export = c_make_oid(GSS_C_NT_COMPOSITE_EXPORT)
